#pragma once

#include "Defs.h"

class XDisplay : public IDisplay
{
	// D3D objects
	IDirect3D8	*			m_lpD3DObject;
	IDirect3DDevice8*		m_lpD3DDevice;
	D3DPRESENT_PARAMETERS	m_d3dDeviceParam;	
	DWORD					m_dwRef;
	//
	// Display settings
	WORD					m_wFlags;
	// Display parameters
	D3DXVECTOR2				m_sRes;
	UINT					m_iDepth;
	D3DDEVTYPE				m_typeDeviceType;
	int						m_iAdapterOrdinal;
	D3DFORMAT				m_formatScreenDepth;
	D3DFORMAT				m_formatTextureDepth;
	D3DTEXTUREFILTERTYPE	m_tftMagTextureFilter;
	D3DTEXTUREFILTERTYPE	m_tftMinTextureFilter;
	D3DTEXTUREFILTERTYPE	m_tftMipTextureFilter;
	//
	// FPS atributes
	float				m_fStartTime;
	float				m_fStopTime;
	FLOAT				m_Frames;
	float				m_fCurrentFPS;
	float				m_fFrameRate;
	int					m_iDesiredFPS;
	char				m_szFPSString[50];
	char				m_szInfoString[512];
	//
	// Camera
	ICamera	*			m_lpCamera;	
	I3DFont				*m_pFPSFont;
	IResourceManager* 	m_lpResManager;
	//
	// Time between two frames
	float				m_fTime;
	// Time from app start
	float				m_fTotalTime;
	// Background color
	D3DCOLOR			m_dwBackgroundColor;
	int					m_iMaxLights;
	
private:
	int			UpdateFPS();
	void		LimitFPS();	
	void		Clean(void);
	void		StartupSettings();
    
public:
	virtual HRESULT			Init(HWND hWnd);
	virtual HRESULT			UpdateBackground();	
	virtual HRESULT			Present();
	virtual HRESULT			RestoreDisplay();
	virtual D3DXVECTOR2*	GetResolution() { return &m_sRes;} 
	virtual ICamera *			GetCamera() {return m_lpCamera;}
	virtual IResourceManager*	GetResourceManager() {return m_lpResManager; }
	virtual LPDIRECT3DDEVICE8	GetDevice() {return m_lpD3DDevice;}
	virtual LPDIRECT3D8			GetD3DObject() {return m_lpD3DObject;}	
	virtual D3DFORMAT			GetTextureFormat() {return m_formatTextureDepth;}
	virtual HRESULT			EnableLight(int iIndex, BOOL bEnable = TRUE);
	virtual HRESULT			SetLight(int iIndex, D3DLIGHT8 * pLight);
	virtual float			GetCurrentFPS() {return m_fCurrentFPS; }

	// Fog methods
	virtual HRESULT	EnableFog(BOOL bEnable);
	virtual HRESULT	SetVertexFog(UINT uFogType, float fDensity, float fStart, float fEnd, D3DCOLOR Color);
	virtual HRESULT	SetPixelFog(UINT uFogType, float fDensity, float fStart, float fEnd, D3DCOLOR Color);
	
public:
	virtual HRESULT		AddRef();
	virtual HRESULT		Release();

	XDisplay(void);
	~XDisplay(void);		
};
