#pragma once
class XResourceManager;
enum MESHTYPE	{BOX, CYLINDER, SPHERE, TEAPOT, TORUS, CUSTOM };

class XMesh : public IMesh
{
	DWORD						m_dwID;
	LPD3DXMESH				m_lpMesh;
	BOOL					m_bVisible;
	D3DXMATRIX				m_TransMat;

	// information for restoring
	char					m_szFilePath[MAX_PATH];
	LPDIRECT3DDEVICE8		m_lpDevice;
	MESHTYPE				m_meshType;
	float					m_fPar1, m_fPar2, m_fPar3;
	UINT					m_uPar4, m_uPar5;
	// number of materials
	DWORD					m_dwNumMat;
	// arrays of textures and materials
	ITexture				**m_ppTextures;
	D3DMATERIAL8			*m_pMaterials;

	DWORD		m_dwRef;

public:
	virtual int						LoadMeshFromFile(LPCSTR szFileName, DWORD dwFlags);	
	virtual int						SetSubsetColor(int iSubSet, D3DCOLOR dwColor);
	virtual int						CreateSphere(float fRadius, UINT uSlices, UINT uStacks);
	virtual int						CreateBox(float fWidth, float fHeight, float fDepth);
	virtual int						CreateCylinder(float fRadius1, float fRadius2, float fLenght, UINT uSlices, UINT uStacks);
	virtual int						CreateTeapot();
	virtual int						CreateTorus(float fInnerRadius, float fOuterRadius, UINT uSides, UINT uRings);
	virtual int						Draw();
	virtual int						Restore();
	virtual int						ReleaseResource();
	virtual LPD3DXMESH				GetMesh()   { return m_lpMesh; }
	virtual LPDIRECT3DDEVICE8		GetDevice() { return m_lpDevice; }
	virtual BOOL					IsVisible() { return m_bVisible; }
	virtual void					Visible(BOOL bVis = TRUE) { m_bVisible = bVis; }
	virtual void					Translate(float fX, float fY, float fZ);
	virtual void					RotateX(float fAngle);
	virtual void					RotateY(float fAngle);
	virtual void					RotateZ(float fAngle);
	virtual void					ScaleXYZ(float fFactor);
	virtual void					ScaleX(float fFactor);
	virtual void					ScaleY(float fFactor);
	virtual void					ScaleZ(float fFactor);
	virtual void					SetID(DWORD dwID) {m_dwID = dwID;}
	virtual DWORD					GetID() {return m_dwID;}
	virtual RES_TYPE				GetType() {return Mesh;}
	virtual	BOOL					operator==(IResource &pRes);

	virtual HRESULT					AddRef();
	virtual HRESULT					Release();
public:
	XMesh(void);
	~XMesh(void);
};
