#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xresourcemanager.h"
#include "xtexture.h"

XTexture::XTexture(void)
{
	m_lpTexture = NULL;
	m_lpDevice = NULL;
	strcpy(m_szFilePath, _S_EMPTY);
	m_dwRef = 0;
	m_dwID = -1;
	m_bSharedTexture = FALSE;
}

XTexture::~XTexture(void)
{		
	ReleaseResource();
	m_lpDevice = NULL;
}

HRESULT		XTexture::AddRef()
{
	return m_dwRef++;
}

HRESULT		XTexture::Release()
{
	DWORD dwID = m_dwID;
	if(m_dwRef > 0)
	{
		m_dwRef--;		
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XTexture object destroyed. ID=%d", dwID);	
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

int		XTexture::LoadTextureFromFile(LPCSTR szFileName)
{		
	return LoadTextureFromFileEx(szFileName, 1, D3DFMT_UNKNOWN, D3DPOOL_MANAGED);	
}

int		XTexture::LoadTextureFromFileEx(LPCSTR szFileName, int iMipLevels, D3DFORMAT formatTextureFormat, D3DPOOL d3dPool)
{	
	IDisplay *dis = NULL;
	ITexture *tex = NULL;
	DWORD dwRet = S_FALSE;
	D3DSURFACE_DESC	texdesc;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();
	
	if(!m_lpTexture) 
	{		
		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}		
		// Add this texture to the manager
		if(!(tex = (ITexture*)dis->GetResourceManager()->AddResource(this, TRUE)))
		{
			// try to create texture
			if(D3D_OK == (dwRet = D3DXCreateTextureFromFileEx(m_lpDevice,  m_szFilePath, 
															  D3DX_DEFAULT, D3DX_DEFAULT, 
															  iMipLevels, 0,
															  formatTextureFormat, d3dPool, 
															  D3DX_DEFAULT, D3DX_DEFAULT, 0, 
															  NULL, NULL, &m_lpTexture)))
			{
				TRACE("Texture '%s' was loaded.", szFileName);
			}
			else 
			{
				TRACE("Texture '%s' wasn't loaded.", szFileName);	
				return dwRet;
			}
		}
		else
		{
			TRACE("Shared texture '%s' loaded.", szFileName);
			m_bSharedTexture = TRUE;
			m_lpTexture = tex->GetTexture();
			if(!m_lpTexture)
			{
				XException exp("Shared texture cannot be created because parent texture is NULL.");
				THROW(exp);
			}
			dwRet = ERROR_SUCCESS;
		}
		// save information to restore		
		m_iMipLevels = iMipLevels;		
		// Get some info about texture		
		if(m_lpTexture)
		{
			dwRet = m_lpTexture->GetLevelDesc(0, &texdesc);
			if(dwRet != S_OK)
			{
				XException exp("GetLevelDesc() function error. Cannot get information about texture.", dwRet);
				THROW(exp);
			}
			// Save info about texture
			m_iWidth = texdesc.Width;
			m_iHeight = texdesc.Height;
			m_formatTextureFormat = texdesc.Format;
			//TRACE("Format is: %d", texdesc.Format);	
		}					
	}
	SAFE_RELEASE(dis);
	return dwRet;	
}


// Tuto metody vola resource manager
int		XTexture::ReleaseResource()
{
	if(m_bSharedTexture)
	{
		TRACE("Texture %s is shared.", m_szFilePath);
	}
	else
	{
		TRACE("Texture %s released.", m_szFilePath);
		SAFE_RELEASE(m_lpTexture); 
	}
	return S_OK;
}

int		XTexture::Restore()
{
	DWORD dwRet = S_FALSE;
	// reinit texture
	if(m_lpDevice && !m_bSharedTexture) 
	{	
		if(D3D_OK == (dwRet = D3DXCreateTextureFromFileEx(m_lpDevice,  m_szFilePath, 0, 0, m_iMipLevels, 
												0, m_formatTextureFormat, D3DPOOL_MANAGED, 
												D3DX_DEFAULT , D3DX_DEFAULT, 0, NULL, NULL, &m_lpTexture)))
		{
			TRACE("Texture '%s' was reloaded.", m_szFilePath);			
		}
		else 
		{
			TRACE("Texture '%s' wasn't reloaded.", m_szFilePath);			
		}		
	}
	return dwRet;
}

BOOL	XTexture::operator==(IResource &pRes)
{
	ITexture *pTex = (ITexture*) &pRes;	
	// Check source
	if(stricmp(pTex->GetTexturePath(), GetTexturePath()) == 0)
	{
		return TRUE;
	}
	return FALSE;
}
