#ifndef BINSTROM_H
  #define BINSTROM_H

  #include "DynZasob.h"
  #include "DynFronta.h"

  template<class T> class CUzel {
		private:
			T m_Data;
			CUzel<T> *m_lpLevy;
			CUzel<T> *m_lpPravy;
		public:
			CUzel() : m_lpLevy(NULL), m_lpPravy(NULL) { }
			CUzel(const T& _Data) : m_Data(_Data), m_lpLevy(NULL), m_lpPravy(NULL) { }

			T VratData() const { return m_Data; }
			CUzel<T>* &VratLevy() { return m_lpLevy; }
			CUzel<T>* &VratPravy() { return m_lpPravy; }
			void NastavLevy(CUzel<T> *_lpLevy) { m_lpLevy = _lpLevy; }
			void NastavPravy(CUzel<T> *_lpPravy) { m_lpPravy = _lpPravy; }
			void NastavData(const T& _Data) { m_Data = _Data; }
  };

  template<class T> class CBinStrom {
		private:
			CUzel<T> *m_lpKoren;
		public:
			CBinStrom() : m_lpKoren(NULL) { }
			~CBinStrom() { if(!JePrazdny()) { SmazStrom(m_lpKoren); } }

			bool VytvorPrazdny();

			bool SmazStrom(CUzel<T>* &_lpUzel);

			bool JePrazdny();

			bool Vloz(const T& _Data);	// vklada do stromu
			void VlozR(const T& _Data);	// vklada do stromu rekurzivne
			bool VlozDoStromu(CUzel<T>* &_lpUzel, const T& _Data);
			void VlozDoStromuR(CUzel<T>* &_lpUzel, const T& _Data);	// vklada rekurzivne do stromu
			
			CUzel<T>* Najdi(const T& _Data, CUzel<T>* &_lpRodic);
			CUzel<T>* NajdiR(const T& _Data);	// vrati ukazatel na nalezeny prvek nebo NULL
			CUzel<T>* Najdi(const T& _Data);	// vrati ukazatel na nalezeny prvek nebo NULL
			CUzel<T>* NajdiVeStromu(CUzel<T> *_lpUzel, const T& _Data, CUzel<T>* &_lpRodic);	// hledani s rodicem
			CUzel<T>* NajdiVeStromuR(CUzel<T> *_lpUzel, const T& _Data);	// rekurzivni verze
			CUzel<T>* NajdiVeStromu(CUzel<T> *_lpUzel, const T& _Data);
			
			void SmazList(CUzel<T>* _lpMazany, CUzel<T>* _lpRodic);	// smaze listovy prvek
			void SmazSJednim(CUzel<T>* _lpMazany, CUzel<T>* _lpRodic);	// smaze prvek s jednim nasledovnikem
			void SmazSeDvema(CUzel<T>* _lpMazany);	// smaze prvek se dvema nasledovniky

			bool Smaz(const T& _Data);	// smaze prvek stromu

			void PruchodPre(CUzel<T>* _lpUzel);
			void PruchodIn(CUzel<T>* _lpUzel);
			void PruchodPost(CUzel<T>* _lpUzel);

			void PruchodPre(CUzel<T> *_lpUzel,void (*operace)(CUzel<T> *_lpParam));
			void PruchodIn(CUzel<T> *_lpUzel,void (*operace)(CUzel<T> *_lpParam));
			void PruchodPost(CUzel<T> *_lpUzel,void (*operace)(CUzel<T> *_lpParam));

			void PruchodStromem1();
			void PruchodStromem2();
			void PruchodStromem3();
			void PruchodStromem4();

			void PruchodStromem1(void (*operace)(CUzel<T> *_lpParam));
			void PruchodStromem2(void (*operace)(CUzel<T> *_lpParam));
			void PruchodStromem3(void (*operace)(CUzel<T> *_lpParam));

			void PruchodStromem5();	// pruchod do sirky
  };

  template<class T> bool CBinStrom<T>::JePrazdny()
  { 
	  return (m_lpKoren == NULL);
  }

  template<class T> bool CBinStrom<T>::VytvorPrazdny()
  {
	  bool bNavrat = false;

	  if(!JePrazdny())
	  {
		  bNavrat = SmazStrom(m_lpKoren);	// smazeme puvodni strom
	  }

	  // nova inicializace promennych se provede jiz v metode SmazStrom()
	  
	  return bNavrat;
  }

  template<class T> bool CBinStrom<T>::SmazStrom(CUzel<T>* &_lpUzel)	// budeme menit ukazatel
  {
	  bool bNavrat = false;

	  if(_lpUzel)
	  {
		  SmazStrom(_lpUzel->VratLevy());
		  SmazStrom(_lpUzel->VratPravy());

		  cout << "Mazu " << (_lpUzel)->VratData() << endl;

		  delete (_lpUzel);
		  _lpUzel = NULL;
		  
		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> bool CBinStrom<T>::VlozDoStromu(CUzel<T>* &_lpUzel, const T& _Data)
  {
	  bool bNavrat = false;

	  // budeme hledat spravne misto pro umisteni uzlu a ukladat si posledni nadrazeny prvek
	  CUzel<T> *lpRodic = NULL, *lpNovy = _lpUzel;

	  while(lpNovy != NULL)
	  {
		  if(lpNovy->VratData() == _Data) { return true; }	// pokud nalezneme prvek, pak koncime
		  else
		  {
			  if(lpNovy->VratData() > _Data)
			  {
				  lpRodic = lpNovy;
				  lpNovy = lpNovy->VratLevy();
			  }
			  else
			  {
				  lpRodic = lpNovy;
				  lpNovy = lpNovy->VratPravy();
			  }
		  }
	  }

	  // nyni mame misto pro novy prvek a tak ho vytvorime
	  lpNovy = new CUzel<T>(_Data);

	  if(lpNovy)
	  {
		  if(lpRodic == NULL)
		  {
			  _lpUzel = lpNovy;	// znaci, ze neexistoval prvni uzel

			  bNavrat = true;
		  }
		  else
		  {
			  if(lpRodic->VratData() > _Data)
			  {
				  lpRodic->NastavLevy(lpNovy);	// zapojime doleva
			  }
			  else
			  {
				  lpRodic->NastavPravy(lpNovy);	// zapojime doprava
			  }

			  bNavrat = true;
		  }
	  }
	  
	  return bNavrat;
  }

  template<class T> void CBinStrom<T>::VlozDoStromuR(CUzel<T>* &_lpUzel, const T& _Data)
  {
	  if(!_lpUzel)
	  {
		  _lpUzel = new CUzel<T>(_Data);	// osetrime prazdny strom
	  }
	  else
	  {
		  if(_Data == _lpUzel->VratData())
		  {
			  return;
		  }

		  if(_lpUzel->VratData() > _Data)
		  {
			  VlozDoStromuR(_lpUzel->VratLevy(), _Data);
		  }
		  else
		  {
			  VlozDoStromuR(_lpUzel->VratPravy(), _Data);
		  }
	  }
  }

  template<class T> bool CBinStrom<T>::Vloz(const T& _Data)
  {
	  return VlozDoStromu(m_lpKoren, _Data);
  }

  template<class T> void CBinStrom<T>::VlozR(const T& _Data)
  {
	  VlozDoStromuR(m_lpKoren, _Data);
  }

  template<class T> CUzel<T>* CBinStrom<T>::NajdiVeStromuR(CUzel<T>* _lpUzel, const T& _Data)
  {
	  CUzel<T> *lpRet = NULL;

	  if(_lpUzel)
	  {
		  if(_lpUzel->VratData() == _Data)
		  {
			  return _lpUzel;
		  }
		  else
		  {
			  if(_lpUzel->VratData() < _Data)
			  {
				  lpRet = NajdiVeStromu(_lpUzel->VratPravy(), _Data);
			  }
			  else
			  {
				  lpRet = NajdiVeStromu(_lpUzel->VratLevy(), _Data);
			  }
		  }
	  }

	  return lpRet;
  } 
  
  template<class T> CUzel<T>* CBinStrom<T>::NajdiVeStromu(CUzel<T>* _lpUzel, const T& _Data)
  {
	  CUzel<T> *lpRet = NULL;

	  while(_lpUzel != NULL)
	  {
		  if(_lpUzel->VratData() == _Data)
		  {
			  lpRet = _lpUzel;
			  break;
		  }
		  else
		  {
			  if(_lpUzel->VratData() < _Data)
			  {
				  _lpUzel = _lpUzel->VratPravy();
			  }
			  else
			  {
				  _lpUzel = _lpUzel->VratLevy();
			  }
		  }
	  }

	  return lpRet;
  }

  template<class T> CUzel<T>* CBinStrom<T>::NajdiVeStromu(CUzel<T> *_lpUzel, const T& _Data, CUzel<T>* &_lpRodic)
  {
	  CUzel<T>* lpRet = NULL;

	  while(_lpUzel != NULL)
	  {
		  if(_lpUzel->VratData() == _Data)
		  {
			  lpRet = _lpUzel;
			  break;
		  }
		  else
		  {
			  _lpRodic = _lpUzel;	// ulozime si rodice

			  if(_lpUzel->VratData() < _Data)
			  {
				  _lpUzel = _lpUzel->VratPravy();
			  }
			  else
			  {
				  _lpUzel = _lpUzel->VratLevy();
			  }
		  }
	  }

	  return lpRet;
  }

  template<class T> CUzel<T>* CBinStrom<T>::NajdiR(const T& _Data)
  {
	  return NajdiVeStromuR(m_lpKoren, _Data);
  }

  template<class T> CUzel<T>* CBinStrom<T>::Najdi(const T& _Data)
  {
	  return NajdiVeStromu(m_lpKoren, _Data);
  }

  template<class T> CUzel<T>* CBinStrom<T>::Najdi(const T& _Data, CUzel<T>* &_lpRodic)
  {
	  _lpRodic = NULL;	// predchudce je NULL
	  return NajdiVeStromu(m_lpKoren, _Data, _lpRodic);
  }

  // smaze prvek _lpMazany, predavame take rodice tohoto prvku
  template<class T> void CBinStrom<T>::SmazList(CUzel<T>* _lpMazany, CUzel<T>* _lpRodic)
  {
	  if(_lpRodic)
	  {
		  // mame-li rodice
		  if(_lpMazany == _lpRodic->VratLevy())
		  {
			  _lpRodic->NastavLevy(NULL);
		  }
		  else
		  {
			  _lpRodic->NastavPravy(NULL);
		  }
	  }
	  else
	  {
		  // mazeme koren stromu
		  m_lpKoren = NULL;
	  }

	  delete _lpMazany;	// kazdopadne mazeme mazany prvek
  }

  template<class T> void CBinStrom<T>::SmazSJednim(CUzel<T>* _lpMazany, CUzel<T>* _lpRodic)
  {
	  // nejprve si uchovame potomka mazaneho prvku - je jen jeden
	  CUzel<T>* lpPotomek = (_lpMazany->VratLevy() == NULL) ? _lpMazany->VratPravy() : _lpMazany->VratLevy();

	  // pokud neexistuje predchudce - budeme mazat koren
	  if(!_lpRodic) { m_lpKoren = lpPotomek; }
	  else
	  {
		  // jinak preskocime mazany ve strukture stromu
	  	  if(_lpMazany == _lpRodic->VratLevy())
		  {
			  _lpRodic->NastavLevy(lpPotomek);
		  }
		  else
		  {
			  _lpRodic->NastavPravy(lpPotomek);
		  }
	  }

	  delete _lpMazany;	// kazdopadne mazeme mazany prvek
  }

  template<class T> void CBinStrom<T>::SmazSeDvema(CUzel<T>* _lpMazany)
  {
	  // uchovame si prvek, kterym nahradime nahrazovany prvek - vezmeme nejlevejsi zprava
	  CUzel<T>* lpNahrazujici = _lpMazany->VratPravy();
	  
	  // soucasne si najdeme i rodice tohoto prvku
	  CUzel<T>* lpRodic = _lpMazany;
	  while(lpNahrazujici->VratLevy())
	  {
		  lpRodic = lpNahrazujici;
		  lpNahrazujici = lpNahrazujici->VratLevy();
	  }  
	  
	  // nyni mame vse potrebne -> prohodime data
	  _lpMazany->NastavData(lpNahrazujici->VratData());

	  // a smazeme nepotrebny prvek
	  if(lpNahrazujici->VratLevy() || lpNahrazujici->VratPravy())
	  {
		  SmazSJednim(lpNahrazujici, lpRodic);
	  }
	  else
	  {
		  SmazList(lpNahrazujici, lpRodic);
	  }
  }

  template<class T> bool CBinStrom<T>::Smaz(const T& _Data)
  {
	  bool bNavrat = false;

	  CUzel<T>* lpMazany = NULL, *lpRodic = NULL;

	  lpMazany = Najdi(_Data, lpRodic);

	  if(lpMazany)
	  {
		  if((lpMazany->VratLevy() == NULL) && (lpMazany->VratPravy() == NULL))
		  {
			  SmazList(lpMazany, lpRodic);
		  }
		  else
		  {
			  if((lpMazany->VratLevy() == NULL) || (lpMazany->VratPravy() == NULL))
			  {
				  SmazSJednim(lpMazany, lpRodic);
			  }
			  else
			  {
				  SmazSeDvema(lpMazany);
			  }
		  }

		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> void CBinStrom<T>::PruchodPre(CUzel<T>* _lpUzel)
  {
	  if(_lpUzel)
	  {
		  cout << _lpUzel->VratData() << " ";
		  PruchodPre(_lpUzel->VratLevy());
		  PruchodPre(_lpUzel->VratPravy());
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem1()
  {
	  PruchodPre(m_lpKoren);
  }

  template<class T> void CBinStrom<T>::PruchodIn(CUzel<T>* _lpUzel)
  {
	  if(_lpUzel)
	  {
		  PruchodIn(_lpUzel->VratLevy());
		  cout << _lpUzel->VratData() << " ";
		  PruchodIn(_lpUzel->VratPravy());
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem2()
  {
	  PruchodIn(m_lpKoren);
  }

  template<class T> void CBinStrom<T>::PruchodPost(CUzel<T>* _lpUzel)
  {
	  if(_lpUzel)
	  {
		  PruchodPost(_lpUzel->VratLevy());
		  PruchodPost(_lpUzel->VratPravy());
		  cout << _lpUzel->VratData() << " ";
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem3()
  {
	  PruchodPost(m_lpKoren);
  }

  template<class T> void CBinStrom<T>::PruchodPre(CUzel<T>* _lpUzel,void (*operace)(CUzel<T> *_lpParam))
  {
	  if(_lpUzel)
	  {
		  operace(_lpUzel);
		  PruchodPre(_lpUzel->VratLevy(), operace);
		  PruchodPre(_lpUzel->VratPravy(), operace);
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem1(void (*operace)(CUzel<T> *_lpParam))
  {
	  PruchodPre(m_lpKoren, operace);
  }

    template<class T> void CBinStrom<T>::PruchodIn(CUzel<T>* _lpUzel,void (*operace)(CUzel<T> *_lpParam))
  {
	  if(_lpUzel)
	  {
		  PruchodIn(_lpUzel->VratLevy(), operace);
		  operace(_lpUzel);
		  PruchodIn(_lpUzel->VratPravy(), operace);
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem2(void (*operace)(CUzel<T> *_lpParam))
  {
	  PruchodIn(m_lpKoren, operace);
  }

    template<class T> void CBinStrom<T>::PruchodPost(CUzel<T>* _lpUzel,void (*operace)(CUzel<T> *_lpParam))
  {
	  if(_lpUzel)
	  {
		  PruchodPre(_lpUzel->VratLevy(), operace);
		  PruchodPre(_lpUzel->VratPravy(), operace);
		  operace(_lpUzel);
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem3(void (*operace)(CUzel<T> *_lpParam))
  {
	  PruchodPost(m_lpKoren, operace);
  }

  template<class T> void CBinStrom<T>::PruchodStromem4()
  {
	  typedef CUzel<T> LPTCUZEL;	// musime si pomoci, abychom mohli pouzit sablonu jako parametr sablony

	  CDynZasob<LPTCUZEL> Zasob;

	  if(!JePrazdny())
	  {
		  Zasob.Vloz(*m_lpKoren);
	  }

	  CUzel<T> Zpracovani;

	  while(!Zasob.JePrazdny())
	  {
		  if(Zasob.Vyjmi(Zpracovani))
		  {
			  cout << Zpracovani.VratData() << " ";
			  if(Zpracovani.VratPravy()) { Zasob.Vloz(*Zpracovani.VratPravy()); }
			  if(Zpracovani.VratLevy()) { Zasob.Vloz(*Zpracovani.VratLevy()); }
		  }
	  }
  }

  template<class T> void CBinStrom<T>::PruchodStromem5()
  {
	  typedef CUzel<T> LPTCUZEL;

	  CDynFronta<LPTCUZEL> Fronta;

	  if(!JePrazdny())	// overime, jestli ve stromu neco je
	  {
		  Fronta.Vloz(*m_lpKoren);
	  }
	  
	  CUzel<T> Zpracovani;

	  while(!Fronta.JePrazdna())
	  {
		  if(Fronta.Vyjmi(Zpracovani))
		  {
			  cout << Zpracovani.VratData() << " ";
			  if(Zpracovani.VratLevy()) { Fronta.Vloz(*Zpracovani.VratLevy()); }	// ukladame zleva doprava
			  if(Zpracovani.VratPravy()) { Fronta.Vloz(*Zpracovani.VratPravy()); }

			  // mohly by nasledovat dalsi nasledovnici u obecnych stromu
		  }
	  }
  }
	
#endif